import qutip  # NOQA : F401
import sax
from ..config import nso
from piel.tools.sax.utils import sax_to_s_parameters_standard_matrix

__all__ = ["sax_to_ideal_qutip_unitary", "standard_s_parameters_to_ideal_qutip_unitary"]


def standard_s_parameters_to_ideal_qutip_unitary(
    s_parameters_standard_matrix: nso.ndarray,
):
    """
    This function converts the calculated S-parameters into a standard Unitary matrix topology so that the shape and
    dimensions of the matrix can be observed.

    I think this means we need to transpose the output of the filtered sax SDense matrix to map it to a QuTip matrix.
    Note that the documentation and formatting of the standard `sax` mapping to a S-parameter standard notation is already in described in piel/piel/sax/utils.py.

    From this stage we can implement a ``QObj`` matrix accordingly and perform simulations accordingly. https://qutip.org/docs/latest/guide/qip/qip-basics.html#unitaries

    For example, a ``qutip`` representation of an s-gate gate would be:

    ..code-block:: python
        import numpy as np
        import qutip

        # S-Gate
        s_gate_matrix = np.array([[1.,   0],
                                 [0., 1.j]])
        s_gate = qutip.Qobj(mat, dims=[[2], [2]])

    In mathematical notation, this S-gate would be written as:

    ..math::
        S = \\begin{bmatrix}
            1 & 0 \\
            0 & i \\
        \\end{bmatrix}

    Args:
        s_parameters_standard_matrix (nso.ndarray): A dictionary of S-parameters in the form of a SDict from `sax`.

    Returns:
        qobj_unitary (qutip.Qobj): A QuTip QObj representation of the S-parameters in a unitary matrix.
    """
    # TODO make a function any SAX input.
    qobj_unitary = qutip.Qobj(s_parameters_standard_matrix)
    return qobj_unitary


def sax_to_ideal_qutip_unitary(sax_input: sax.SType):
    """
    This function converts the calculated S-parameters into a standard Unitary matrix topology so that the shape and
    dimensions of the matrix can be observed.

    I think this means we need to transpose the output of the filtered sax SDense matrix to map it to a QuTip matrix.
    Note that the documentation and formatting of the standard `sax` mapping to a S-parameter standard notation is already in described in piel/piel/sax/utils.py.

    From this stage we can implement a ``QObj`` matrix accordingly and perform simulations accordingly. https://qutip.org/docs/latest/guide/qip/qip-basics.html#unitaries

    For example, a ``qutip`` representation of an s-gate gate would be:

    ..code-block:: python

        import numpy as np
        import qutip
        # S-Gate
        s_gate_matrix = np.array([[1.,   0],
                                 [0., 1.j]])
        s_gate = qutip.Qobj(mat, dims=[[2], [2]])

    In mathematical notation, this S-gate would be written as:

    ..math::

        S = \\begin{bmatrix}
            1 & 0 \\
            0 & i \\
        \\end{bmatrix}

    Args:
        sax_input (sax.SType): A dictionary of S-parameters in the form of a SDict from `sax`.

    Returns:

    """
    # TODO make a function any SAX input.
    (
        s_parameters_standard_matrix,
        input_ports_index_tuple_order,
    ) = sax_to_s_parameters_standard_matrix(sax_input)
    qobj_unitary = standard_s_parameters_to_ideal_qutip_unitary(
        s_parameters_standard_matrix
    )
    return qobj_unitary



from .photonic.straight_waveguide import waveguide
from .photonic.mmi2x2 import mmi2x2_50_50


# Default model dictionary library that can be overwritten for specific modelling applications.
__default_models_dictionary__ = {
    "bend_euler": waveguide,
    "mmi2x2": mmi2x2_50_50,
    "straight": waveguide,
}


def get_default_models() -> dict:
    """
    Returns the default models dictionary.

    Returns:
        dict: Default models dictionary.
    """
    return __default_models_dictionary__


__all__ = [
    "get_default_models",
]



"""
We create a set of parameters that can be used throughout the project for optimisation.

The numerical solver is normally delegated for as `numpy` but there are cases where a much faster solver is desired, and where different functioanlity is required. For example, `sax` uses `JAX` for its numerical solver. In this case, we will create a global numerical solver that we can use throughout the project, and that can be extended and solved accordingly for the particular project requirements.
"""
import pathlib
import sys
import types


if "jax" in sys.modules:
    import jax.numpy as jnp

    numerical_solver = jnp
elif "numpy" in sys.modules:
    import numpy

    numerical_solver = numpy
else:
    import numpy

    numerical_solver = numpy

nso = numerical_solver
piel_path_types = str | pathlib.Path | types.ModuleType

__all__ = [
    "numerical_solver",
    "nso",
    "piel_path_types",
]



from ...config import nso


def heat_transfer_1d_W(
    thermal_conductivity_fit, temperature_range_K, cross_sectional_area_m2, length_m
) -> float:
    thermal_conductivity_integral_area = nso.trapz(
        thermal_conductivity_fit, temperature_range_K
    )
    return cross_sectional_area_m2 * thermal_conductivity_integral_area / length_m



import pandas as pd
import matplotlib.pyplot as plt

__all__ = [
    "plot_simple_multi_row",
    "plot_multi_row",
]


def plot_simple_multi_row(
    data: pd.DataFrame,
    x_axis_column_name: str = "t",
    row_list: list | None = None,
    y_axis_title_list: list | None = None,
    x_axis_title: str | None = None,
):
    """
    Plot multiple rows of data on the same plot. Each row is a different line. Each row is a different y axis. The x
    axis is the same for all rows. The y axis title is the same for all rows.

    Args:
        data (pd.DataFrame): Data to plot.
        x_axis_column_name (str, optional): Column name of the x axis. Defaults to "t".
        row_list (list, optional): List of column names to plot. Defaults to None.
        y_axis_title_list (list, optional): List of y axis titles. Defaults to None.
        x_axis_title (str, optional): Title of the x axis. Defaults to None.

    Returns:
        plt: Matplotlib plot.
    """
    x = data[x_axis_column_name]
    y_array = []

    if y_axis_title_list is None:
        y_axis_title_list = row_list

    row_amount = len(row_list)
    for row_name in row_list:
        y_array.append(data[row_name])

    fig, axes = plt.subplots(row_amount, 1, sharex=True)

    for i in range(len(row_list)):
        axes[i].plot(x, y_array[i])
        axes[i].grid(True)
        axes[i].set(ylabel=y_axis_title_list[i])

    # TODO Xaxis title
    # TODO align all ytitles

    return plt


def plot_multi_row(
    data: pd.DataFrame,
):
    pass
    # from bokeh.plotting import figure, show
    # from bokeh.layouts import column
    #
    # p = figure(
    #     width=800,
    #     height=300,
    #     title="",
    #     tools="",
    #     toolbar_location=None,
    #     match_aspect=True,
    #     y_range=[0, 1],
    # )
    # p2 = figure(width=800, height=300, x_range=p.x_range)
    # p3 = figure(width=800, height=300, x_range=p.x_range)
    #
    # p.line(
    #     mzi2x2_simple_simulation_data.t / 1000,
    #     mzi2x2_simple_simulation_data.output_amplitude_array_0_abs,
    # )
    # # color="navy", alpha=0.4, line_width=4)
    #
    # p.line(
    #     mzi2x2_simple_simulation_data.t / 1000,
    #     mzi2x2_simple_simulation_data.output_amplitude_array_1_abs,
    # )
    # # color="navy", alpha=0.4, line_width=4)
    #
    # p2.line(
    #     mzi2x2_simple_simulation_data.t / 1000,
    #     mzi2x2_simple_simulation_data.output_amplitude_array_0_phase_deg,
    # )
    # # color="navy", alpha=0.4, line_width=4)
    #
    # p2.line(
    #     mzi2x2_simple_simulation_data.t / 1000,
    #     mzi2x2_simple_simulation_data.output_amplitude_array_1_phase_deg,
    # )
    #
    # p3.line(
    #     mzi2x2_simple_simulation_data.t / 1000,
    #     mzi2x2_simple_simulation_data.phase,
    # )
    # # color="navy", alpha=0.4, line_width=4)
    #
    # # color="navy", alpha=0.4, line_width=4)
    #
    # # show(p)
    # # layout = gridplot([[p], [p2]])
    # return show(column(p, p2, p3))
